/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.stub;

import java.util.LinkedHashMap;
import java.util.Map;

import ru.ispras.fortress.data.DataType;
import ru.ispras.fortress.data.DataTypeId;
import ru.ispras.fortress.data.types.bitvector.BitVector;
import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.testbase.TestBaseQuery;
import ru.ispras.testbase.TestData;
import ru.ispras.testbase.generator.Utils;

final class TdpZero extends TdpBase {
  public static final String NAME = "zero";

  public TdpZero() {
    super(NAME);
  }

  @Override
  protected TestData generateData(final TestBaseQuery query) {
    final Map<String, Node> unknowns = Utils.extractUnknown(query);
    final Map<String, Node> outputData = new LinkedHashMap<>();

    for (Map.Entry<String, Node> e : unknowns.entrySet()) {
      final String name = e.getKey();
      final DataType type = e.getValue().getDataType();

      final Node value;
      if (DataTypeId.BIT_VECTOR == type.getTypeId()) {
        value = NodeValue.newBitVector(BitVector.newEmpty(type.getSize()));
      } else {
        throw new IllegalArgumentException(String.format(
          "The %s variable has unupported type: %s", name, type));
      }

      outputData.put(name, value);
    }

    return new TestData(outputData);
  }
}
